@@ -97,6 +97,12 @@
           (fw)->flags.common.s.has_stippled_title = !!(x)
 #define SETM_HAS_STIPPLED_TITLE(fw,x) \
           (fw)->flag_mask.common.s.has_stippled_title = !!(x)
+#define ICON_OVERRIDE_MODE(fw)  \
+          ((fw)->flags.common.s.icon_override)
+#define SET_ICON_OVERRIDE_MODE(fw,x)  \
+          (fw)->flags.common.s.icon_override = !!(x)
+#define SETM_ICON_OVERRIDE_MODE(fw,x) \
+          (fw)->flag_mask.common.s.icon_override = !!(x)
 #define IS_ICON_STICKY(fw)     ((fw)->flags.common.s.is_icon_sticky)
 #define SET_ICON_STICKY(fw,x)  (fw)->flags.common.s.is_icon_sticky = !!(x)
 #define SETM_ICON_STICKY(fw,x) \
@@ -154,12 +160,17 @@
                                ((fw)->flags.common.s.has_mwm_override)
 #define HAS_OVERRIDE_SIZE_HINTS(fw)  \
                                ((fw)->flags.common.s.has_override_size)
+#define DO_ICONIFY_WINDOW_GROUPS(fw)  \
+                               ((fw)->flags.common.s.do_iconify_window_groups)
 #define DO_IGNORE_GNOME_HINTS(fw)  \
                                ((fw)->flags.common.s.do_ignore_gnome_hints)
 #define DO_IGNORE_RESTACK(fw)  \
                                ((fw)->flags.common.s.do_ignore_restack)
-#define IS_FIXED(fw)  \
-                               ((fw)->flags.common.s.is_fixed)
+#define DO_IGNORE_ICON_BOXES(fw)  \
+                               ((fw)->flags.common.s.do_ignore_icon_boxes)
+#define DO_USE_WINDOW_GROUP_HINT(fw)  \
+                             ((fw)->flags.common.s.do_use_window_group_hint)
+#define IS_FIXED(fw)           ((fw)->flags.common.s.is_fixed)
 #define SET_FIXED(fw,x)        (fw)->flags.common.s.is_fixed = !!(x)
 #define SETM_FIXED(fw,x)       (fw)->flag_mask.common.s.is_fixed = !!(x)
 #define HAS_DEPRESSABLE_BORDER(fw) ((fw)->flags.common.s.has_depressable_border)
@@ -180,12 +191,18 @@
 #define HAS_BORDER(fw)         ((fw)->flags.has_border)
 #define SET_HAS_BORDER(fw,x)   (fw)->flags.has_border = !!(x)
 #define SETM_HAS_BORDER(fw,x)  (fw)->flag_mask.has_border = !!(x)
+#define HAS_ICON_CHANGED(fw)         ((fw)->flags.has_icon_changed)
+#define SET_HAS_ICON_CHANGED(fw,x)   (fw)->flags.has_icon_changed = !!(x)
+#define SETM_HAS_ICON_CHANGED(fw,x)  (fw)->flag_mask.has_icon_changed = !!(x)
 #define HAS_TITLE(fw)          ((fw)->flags.has_title)
 #define SET_HAS_TITLE(fw,x)    (fw)->flags.has_title = !!(x)
 #define SETM_HAS_TITLE(fw,x)   (fw)->flag_mask.has_title = !!(x)
 #define IS_MAPPED(fw)          ((fw)->flags.is_mapped)
 #define SET_MAPPED(fw,x)       (fw)->flags.is_mapped = !!(x)
 #define SETM_MAPPED(fw,x)      (fw)->flag_mask.is_mapped = !!(x)
+#define IS_DECOR_CHANGED(fw)     ((fw)->flags.is_decor_changed)
+#define SET_DECOR_CHANGED(fw,x)  (fw)->flags.is_decor_changed = !!(x)
+#define SETM_DECOR_CHANGED(fw,x) (fw)->flag_mask.is_decor_changed = !!(x)
 #define IS_ICON_FONT_LOADED(fw)     ((fw)->flags.is_icon_font_loaded)
 #define SET_ICON_FONT_LOADED(fw,x)  (fw)->flags.is_icon_font_loaded = !!(x)
 #define SETM_ICON_FONT_LOADED(fw,x) (fw)->flag_mask.is_icon_font_loaded = !!(x)
@@ -233,6 +250,9 @@
 #define IS_PLACED_WB3(fw)      ((fw)->flags.is_placed_wb3)
 #define SET_PLACED_WB3(fw,x)   (fw)->flags.is_placed_wb3 = !!(x)
 #define SETM_PLACED_WB3(fw,x)  (fw)->flag_mask.is_placed_wb3 = !!(x)
+#define IS_PLACED_BY_FVWM(fw)      ((fw)->flags.is_placed_by_fvwm)
+#define SET_PLACED_BY_FVWM(fw,x)   (fw)->flags.is_placed_by_fvwm = (x)
+#define SETM_PLACED_BY_FVWM(fw,x)  (fw)->flag_mask.is_placed_by_fvwm = (x)
 #define IS_SCHEDULED_FOR_DESTROY(fw)    ((fw)->flags.is_scheduled_for_destroy)
 #define SET_SCHEDULED_FOR_DESTROY(fw,x) \
           (fw)->flags.is_scheduled_for_destroy = !!(x)
@@ -255,12 +275,21 @@
 #define IS_SIZE_INC_SET(fw)     ((fw)->flags.is_size_inc_set)
 #define SET_SIZE_INC_SET(fw,x)  (fw)->flags.is_size_inc_set = !!(x)
 #define SETM_SIZE_INC_SET(fw,x) (fw)->flag_mask.is_size_inc_set = !!(x)
+#define IS_STYLE_DELETED(fw)     ((fw)->flags.is_style_deleted)
+#define SET_STYLE_DELETED(fw,x)  (fw)->flags.is_style_deleted = !!(x)
+#define SETM_STYLE_DELETED(fw,x) (fw)->flag_mask.is_style_deleted = !!(x)
 #define IS_VIEWPORT_MOVED(fw)  ((fw)->flags.is_viewport_moved)
 #define SET_VIEWPORT_MOVED(fw,x) \
           (fw)->flags.is_viewport_moved = !!(x)
 #define SETM_VIEWPORT_MOVED(fw,x) \
           (fw)->flag_mask.is_viewport_moved = !!(x)
 #define IS_VIEWPORT_MOVED(fw)  ((fw)->flags.is_viewport_moved)
+#define IS_FOCUS_CHANGE_BROADCAST_PENDING(fw) \
+          ((fw)->flags.is_focus_change_broadcast_pending)
+#define SET_FOCUS_CHANGE_BROADCAST_PENDING(fw,x) \
+          (fw)->flags.is_focus_change_broadcast_pending = !!(x)
+#define SETM_FOCUS_CHANGE_BROADCAST_PENDING(fw,x) \
+          (fw)->flag_mask.is_focus_change_broadcast_pending = !!(x)
 #define IS_FULLY_VISIBLE(fw)   ((fw)->flags.is_fully_visible)
 #define SET_FULLY_VISIBLE(fw,x) \
           (fw)->flags.is_fully_visible = !!(x)
@@ -284,6 +313,11 @@
           (fw)->flags.is_window_being_moved_opaque = !!(x)
 #define SETM_WINDOW_BEING_MOVED_OPAQUE(fw,x) \
           (fw)->flag_mask.is_window_being_moved_opaque = !!(x)
+#define IS_WINDOW_BORDER_DRAWN(fw)     ((fw)->flags.is_window_border_drawn)
+#define SET_WINDOW_BORDER_DRAWN(fw,x)  \
+          (fw)->flags.is_window_border_drawn = !!(x)
+#define SETM_WINDOW_BORDER_DRAWN(fw,x) \
+          (fw)->flag_mask.is_window_border_drawn = !!(x)
 #define IS_WINDOW_FONT_LOADED(fw)     ((fw)->flags.is_window_font_loaded)
 #define SET_WINDOW_FONT_LOADED(fw,x)  (fw)->flags.is_window_font_loaded = !!(x)
 #define SETM_WINDOW_FONT_LOADED(fw,x) \
@@ -310,6 +344,18 @@
           (fw)->flags.using_default_icon_font = !!(x)
 #define SETM_USING_DEFAULT_ICON_FONT(fw,x) \
           (fw)->flag_mask.using_default_icon_font = !!(x)
+#define WAS_ICON_HINT_PROVIDED(fw) \
+          ((fw)->flags.was_icon_hint_provided)
+#define SET_WAS_ICON_HINT_PROVIDED(fw,x) \
+          (fw)->flags.was_icon_hint_provided = (x)
+#define SETM_WAS_ICON_HINT_PROVIDED(fw,x) \
+          (fw)->flag_mask.was_icon_hint_provided = (x)
+#define WAS_ICON_NAME_PROVIDED(fw) \
+          ((fw)->flags.was_icon_name_provided)
+#define SET_WAS_ICON_NAME_PROVIDED(fw,x) \
+          (fw)->flags.was_icon_name_provided = (x)
+#define SETM_WAS_ICON_NAME_PROVIDED(fw,x) \
+          (fw)->flag_mask.was_icon_name_provided = (x)
 
 
 #endif /* _WINDOW_FLAGS_ */
