@@ -20,67 +20,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* Changed 12/20/98 Dan Espen:
-
- - Removed form limitations including:
-   number of lines in a form.
-   number of items in a form.
-   number of items in a line.
-   number of selections in a choice.
-   number of commands on a button.
-   Colors can be changed anywhere in a form.
-   Fonts can be changed anywhere in a form.
-
- - Changed the general organization of the module to match FvwmAnimate.
-   See comments in FvwmAnimate to see what it mimicks.
-   Some parts of this module have comments containing "FvwmAnimate"
-   are common module/macro candidates.
-   Changed debugging technique to match FvwmAnimate with the
-   additional ability to Debug to File.
-
- - Configurability updates:
-   Form appearance can be configured globaly:
-   Form defaults are read from .FvwmForm.
-   There is a built in Default setting/saving dialogue.
-   Forms can be read in directly from a file.
-   The file is the alias with a leading dot.
-   The file is in $HOME or the system configuration directory.
-   Comes with forms installed in the system configuration directory.
-   Variable substitution throught the form input from invocation command line.
-
- - Operability:
-   You can tab to previous input field with ^P, Up arrow, shift tab.
-
- - This module now has a configuration proceedure:
-   AddToMenu "Module-Popup" "FvwmForm Defaults" FvwmForm FvwmForm-Form
-
- - Use FvwmAnimate command parsing.
-   The part of the command after the module name is no longer case sensitive.
-   Use command tables instead of huge "else if".
-
- - Misc:
-   Avoid core when choice not preceeded by a selection.
-   Rename union member "select" so it doesn't conflict with the function.
-   You can now control vertical spacing on text.  By default text is spaced
-   vertically the way you would want it for buttons.
-   This is for compatibility.  Now you can change the spacing to zero as
-   you might want for a help panel.
-   A button can execute a synchronous shell command.  The first use I
-   put this is a form that writes its new definition to a file and
-   reinvokes itself.
-   Use SendText instead of writes to pipe.
-   Changed button press-in effect from 1 sec to .1 sec.  Didn't seem to
-   do anything on a slow machine...
-   Added preload arg, and Map, Stop and UnMap commands for fast forms.
-   (FvwmForm is now parsing commands during form display.)
-   Add "Message" command, display "Error" and "String" messages from fvwm.
-   Removed CopyNString, strdup replaces it.
-
-   Moved ParseCommand, ReadXServer to separate file.
-   Add UseData.
-   Remove DefineMe.
-
- */
 #include "config.h"
 
 #include <stdio.h>
@@ -107,10 +46,40 @@
 
 #include <libs/Picture.h>               /* for InitPictureCMap */
 #include "libs/Colorset.h"               /* for InitPictureCMap */
+#include "libs/FScreen.h"
+#include "libs/FShape.h"
 
-#define IamTheMain 1                    /* in FvwmForm.h, chg extern to "" */
 #include "FvwmForm.h"                   /* common FvwmForm stuff */
-#undef IamTheMain
+
+/* globals that are exported, keep in sync with externs in FvwmForm.h */
+Form cur_form;                   /* current form */
+
+/* Link list roots */
+Item *root_item_ptr;             /* pointer to root of item list */
+Line root_line = {&root_line,    /* ->next */
+                  0,             /* number of items */
+                  L_CENTER,0,0,  /* justify, size x/y */
+                  0,             /* current array size */
+                  0};            /* items ptr */
+Line *cur_line = &root_line;            /* curr line in parse rtns */
+char preload_yorn='n';           /* init to non-preload */
+Item *item;                             /* current during parse */
+Item *cur_sel, *cur_button;             /* current during parse */
+Display *dpy;
+int fd_x;                  /* fd for X connection */
+Window root, ref;
+int screen;
+char *color_names[4];
+char bg_state = 'd';                    /* in default state */
+  /* d = default state */
+  /* s = set by command (must be in "d" state for first "back" cmd to set it) */
+  /* u = used (color allocated, too late to accept "back") */
+char endDefaultsRead = 'n';
+char *font_names[3];
+char *screen_background_color;
+char *MyName;
+int MyNameLen;
+int Channel[2];
 
 /* Font/color stuff
    The colors are:
@@ -190,9 +159,16 @@ typedef struct CommandTable
   char *name;
   void (*function)(char *);
 } ct;
+static void ct_ActivateOnPress(char *);
 static void ct_Back(char *);
 static void ct_Button(char *);
 static void ct_ButtonFont(char *);
+static void ct_ButtonInPointer(char *);
+static void ct_ButtonInPointerBack(char *);
+static void ct_ButtonInPointerFore(char *);
+static void ct_ButtonPointer(char *);
+static void ct_ButtonPointerBack(char *);
+static void ct_ButtonPointerFore(char *);
 static void ct_Choice(char *);
 static void ct_Command(char *);
 static void ct_Font(char *);
@@ -204,10 +180,14 @@ static void ct_ItemBack(char *);
 static void ct_ItemFore(char *);
 static void ct_Line(char *);
 static void ct_Message(char *);
+static void ct_Geometry(char *);
 static void ct_Position(char *);
 static void ct_Read(char *);
 static void ct_Selection(char *);
 static void ct_Text(char *);
+static void ct_InputPointer(char *);
+static void ct_InputPointerBack(char *);
+static void ct_InputPointerFore(char *);
 static void ct_Title(char *);
 static void ct_UseData(char *);
 static void ct_padVText(char *);
@@ -218,17 +198,28 @@ static void ct_ItemColorset(char *);
 /* Must be in Alphabetic order (caseless) */
 static struct CommandTable ct_table[] =
 {
+  {"ActivateOnPress",ct_ActivateOnPress},
   {"Back",ct_Back},
   {"Button",ct_Button},
   {"ButtonFont",ct_ButtonFont},
+  {"ButtonInPointer",ct_ButtonInPointer},
+  {"ButtonInPointerBack",ct_ButtonInPointerBack},
+  {"ButtonInPointerFore",ct_ButtonInPointerFore},
+  {"ButtonPointer",ct_ButtonPointer},
+  {"ButtonPointerBack",ct_ButtonPointerBack},
+  {"ButtonPointerFore",ct_ButtonPointerFore},
   {"Choice",ct_Choice},
   {"Colorset",ct_Colorset},
   {"Command",ct_Command},
   {"Font",ct_Font},
   {"Fore",ct_Fore},
+  {"Geometry",ct_Geometry},
   {"GrabServer",ct_GrabServer},
   {"Input",ct_Input},
   {"InputFont",ct_InputFont},
+  {"InputPointer",ct_InputPointer},
+  {"InputPointerBack",ct_InputPointerBack},
+  {"InputPointerFore",ct_InputPointerFore},
   {"ItemBack",ct_ItemBack},
   {"ItemColorset",ct_ItemColorset},
   {"ItemFore",ct_ItemFore},
@@ -246,12 +237,22 @@ static struct CommandTable ct_table[] =
    read before the other form defining commands. */
 static struct CommandTable def_table[] =
 {
+  {"ActivateOnPress",ct_ActivateOnPress},
   {"Back",ct_Back},
   {"ButtonFont",ct_ButtonFont},
+  {"ButtonInPointer",ct_ButtonInPointer},
+  {"ButtonInPointerBack",ct_ButtonInPointerBack},
+  {"ButtonInPointerFore",ct_ButtonInPointerFore},
+  {"ButtonPointer",ct_ButtonPointer},
+  {"ButtonPointerBack",ct_ButtonPointerBack},
+  {"ButtonPointerFore",ct_ButtonPointerFore},
   {"Colorset",ct_Colorset},
   {"Font",ct_Font},
   {"Fore",ct_Fore},
   {"InputFont",ct_InputFont},
+  {"InputPointer",ct_InputPointer},
+  {"InputPointerBack",ct_InputPointerBack},
+  {"InputPointerFore",ct_InputPointerFore},
   {"ItemBack",ct_ItemBack},
   {"ItemColorset",ct_ItemColorset},
   {"ItemFore",ct_ItemFore},
@@ -309,7 +310,12 @@ static void ParseConfigLine(char *buf)
     buf[strlen(buf)-1] = '\0';	/* strip off \n */
   }
 
-  /* This used to be case sensitive */
+  if (strncasecmp(buf, XINERAMA_CONFIG_STRING,
+		  sizeof(XINERAMA_CONFIG_STRING)-1) == 0)
+  {
+    FScreenConfigureModule(buf + sizeof(XINERAMA_CONFIG_STRING)-1);
+    return;
+  }
   if (strncasecmp(buf, "Colorset", 8) == 0) {
     LoadColorset(&buf[8]);
     return;
@@ -360,6 +366,36 @@ static void AddToLine(Item *newItem)
 
 /* All the functions starting with "ct_" (command table) are called thru
    their function pointers. Arg 1 is always the rest of the command. */
+static void ct_ActivateOnPress(char *cp)
+{
+  /* arg can be "on", "off", "0", 1", "true", "false", "" */
+  char option[6];
+  int i,j;
+  if (strlen(cp) > 5) {
+    fprintf(stderr,"%s: arg for ActivateOnPress (%s) too long\n",
+            MyName+1,cp);
+    return;
+  }
+  for (i=0,j=0;i<strlen(cp);i++) {
+    if (cp[i] != ' ') {                   /* remove any spaces */
+      option[j++]=tolower(cp[i]);
+    }
+  }
+  option[j]=0;
+  if (strcmp(option,"on") == 0
+      || strcmp(option,"true") == 0
+      || strcmp(option,"1") == 0
+      || (cp == 0 || strcmp(option,"")) == 0) {
+    CF.activate_on_press = 1;
+  } else if (strcmp(option,"off") == 0
+             || strcmp(option,"false") == 0
+             || strcmp(option,"0") == 0) {
+    CF.activate_on_press = 0;
+  } else {
+    fprintf(stderr,"%s: arg for ActivateOnPress (%s/%s) invalid\n",
+            MyName+1,option,cp);
+  }
+}
 static void ct_GrabServer(char *cp)
 {
   CF.grab_server = 1;
@@ -372,16 +408,54 @@ static void ct_Position(char *cp)
 {
   CF.have_geom = 1;
   CF.gx = atoi(cp);
+  CF.xneg = 0;
+  if (CF.gx < 0)
+  {
+    CF.gx = -1 - CF.gx;
+    CF.xneg = 1;
+  }
   while (!isspace((unsigned char)*cp)) cp++;
   while (isspace((unsigned char)*cp)) cp++;
   CF.gy = atoi(cp);
-  myfprintf((stderr, "Position @ (%d, %d)\n", CF.gx, CF.gy));
+  CF.yneg = 0;
+  if (CF.gy < 0)
+  {
+    CF.gy = -1 - CF.gy;
+    CF.yneg = 1;
+  }
+  myfprintf((stderr, "Position @ (%c%d%c%d)\n",
+	     (CF.xneg)?'-':'+',CF.gx, (CF.yneg)?'-':'+',CF.gy));
+}
+static void ct_Geometry(char *cp)
+{
+  int x = 0;
+  int y = 0;
+  int flags;
+  unsigned int dummy;
+
+  while (isspace(*cp))
+    cp++;
+  flags = FScreenParseGeometry(cp, &x, &y, &dummy, &dummy);
+  if (flags&XValue)
+  {
+    CF.have_geom = 1;
+    CF.gx = x;
+    CF.xneg = (flags&XNegative);
+  }
+  if (flags&YValue)
+  {
+    CF.have_geom = 1;
+    CF.gy = y;
+    CF.yneg = (flags&YNegative);
+  }
+  myfprintf((stderr, "Geometry @ (%c%d%c%d)\n",
+	     (CF.xneg)?'-':'+',CF.gx, (CF.yneg)?'-':'+',CF.gy));
 }
 static void ct_Fore(char *cp)
 {
   if (color_names[c_fg])
     free(color_names[c_fg]);
-  color_names[c_fg] = strdup(cp);
+  color_names[c_fg] = safestrdup(cp);
   colorset = -1;
   myfprintf((stderr, "ColorFore: %s\n", color_names[c_fg]));
 }
@@ -389,12 +463,12 @@ static void ct_Back(char *cp)
 {
   if (color_names[c_bg])
     free(color_names[c_bg]);
-  color_names[c_bg] = strdup(cp);
+  color_names[c_bg] = safestrdup(cp);
   if (bg_state == 'd')
   {
     if (screen_background_color)
       free(screen_background_color);
-    screen_background_color = strdup(color_names[c_bg]);
+    screen_background_color = safestrdup(color_names[c_bg]);
     bg_state = 's';                     /* indicate set by command */
   }
   colorset = -1;
@@ -410,7 +484,7 @@ static void ct_ItemFore(char *cp)
 {
   if (color_names[c_item_fg])
     free(color_names[c_item_fg]);
-  color_names[c_item_fg] = strdup(cp);
+  color_names[c_item_fg] = safestrdup(cp);
   itemcolorset = -1;
   myfprintf((stderr, "ColorItemFore: %s\n", color_names[c_item_fg]));
 }
@@ -418,7 +492,7 @@ static void ct_ItemBack(char *cp)
 {
   if (color_names[c_item_bg])
     free(color_names[c_item_bg]);
-  color_names[c_item_bg] = strdup(cp);
+  color_names[c_item_bg] = safestrdup(cp);
   itemcolorset = -1;
   myfprintf((stderr, "ColorItemBack: %s\n", color_names[c_item_bg]));
 }
@@ -431,23 +505,82 @@ static void ct_Font(char *cp)
 {
   if (font_names[f_text])
     free(font_names[f_text]);
-  font_names[f_text] = strdup(cp);
+  font_names[f_text] = safestrdup(cp);
   myfprintf((stderr, "Font: %s\n", font_names[f_text]));
 }
 static void ct_ButtonFont(char *cp)
 {
   if (font_names[f_button])
     free(font_names[f_button]);
-  font_names[f_button] = strdup(cp);
+  font_names[f_button] = safestrdup(cp);
   myfprintf((stderr, "ButtonFont: %s\n", font_names[f_button]));
 }
+static void ct_ButtonInPointer(char *cp)
+{
+  int cursor;
+  cursor=fvwmCursorNameToIndex(cp);
+  if (cursor == -1) {
+    fprintf(stderr,"ButtonInPointer: invalid cursor name %s\n",cp);
+  } else {
+    CF.pointer[button_in_pointer] = XCreateFontCursor(dpy,cursor);
+  }
+}
+static void ct_ButtonPointer(char *cp)
+{
+  int cursor;
+  cursor=fvwmCursorNameToIndex(cp);
+  if (cursor == -1) {
+    fprintf(stderr,"ButtonPointer: invalid cursor name %s\n",cp);
+  } else {
+    CF.pointer[button_pointer] = XCreateFontCursor(dpy,cursor);
+  }
+}
 static void ct_InputFont(char *cp)
 {
   if (font_names[f_input])
     free(font_names[f_input]);
-  font_names[f_input] = strdup(cp);
+  font_names[f_input] = safestrdup(cp);
   myfprintf((stderr, "InputFont: %s\n", font_names[f_input]));
 }
+static void ct_InputPointer(char *cp)
+{
+  int cursor;
+  cursor=fvwmCursorNameToIndex(cp);
+  if (cursor == -1) {
+    fprintf(stderr,"InputPointer: invalid cursor name %s\n",cp);
+  } else {
+    CF.pointer[input_pointer] = XCreateFontCursor(dpy,cursor);
+  }
+}
+/* process buttons using an index to the ___ array */
+static void color_arg(Ptr_color *color_cell, char *color_name);
+static void color_arg(Ptr_color *color_cell, char *color_name) {
+  if (color_name && *color_name) {
+    color_cell->pointer_color.pixel = GetColor(color_name);
+    color_cell->used=1;
+  }
+}
+
+/* arg is a color name, alloc the color */
+static void ct_ButtonInPointerBack(char *cp) {
+  color_arg(&CF.p_c[button_in_back],cp);
+}
+static void ct_ButtonInPointerFore(char *cp) {
+  color_arg(&CF.p_c[button_in_fore],cp);
+}
+static void ct_ButtonPointerBack(char *cp) {
+  color_arg(&CF.p_c[button_back],cp);
+}
+static void ct_ButtonPointerFore(char *cp) {
+  color_arg(&CF.p_c[button_fore],cp);
+}
+static void ct_InputPointerBack(char *cp) {
+  color_arg(&CF.p_c[input_back],cp);
+}
+static void ct_InputPointerFore(char *cp) {
+  color_arg(&CF.p_c[input_fore],cp);
+}
+
 static void ct_Line(char *cp)
 {
   /* malloc new line */
@@ -596,11 +729,11 @@ static void AssignDrawTable(Item *adt_item)
     last_dt->dt_next = new_dt;          /* link old to new */
   }
 
-  new_dt->dt_font_name = strdup(match_font);
-  new_dt->dt_color_names[c_fg]      = strdup(match_text_fore);
-  new_dt->dt_color_names[c_bg]      = strdup(match_text_back);
-  new_dt->dt_color_names[c_item_fg] = strdup(match_item_fore);
-  new_dt->dt_color_names[c_item_bg] = strdup(match_item_back);
+  new_dt->dt_font_name = safestrdup(match_font);
+  new_dt->dt_color_names[c_fg]      = safestrdup(match_text_fore);
+  new_dt->dt_color_names[c_bg]      = safestrdup(match_text_back);
+  new_dt->dt_color_names[c_item_fg] = safestrdup(match_item_fore);
+  new_dt->dt_color_names[c_item_bg] = safestrdup(match_item_back);
   new_dt->dt_used = 0;                  /* show nothing allocated */
   new_dt->dt_font_struct = GetFontOrFixed(dpy, new_dt->dt_font_name);
   new_dt->dt_font = new_dt->dt_font_struct->fid;
@@ -682,7 +815,7 @@ static void ct_Input(char *cp)
   item->input.size = atoi(cp);
   while (!isspace((unsigned char)*cp)) cp++;
   while (isspace((unsigned char)*cp)) cp++;
-  item->input.init_value = strdup("");          /* init */
+  item->input.init_value = safestrdup("");          /* init */
   if (*cp == '\"') {
     free(item->input.init_value);
     item->input.init_value = CopyQuotedString(++cp);
@@ -796,7 +929,8 @@ static void PutDataInForm(char *cp)
         strcpy(item->input.init_value,cp); /* new initial value in field */
 	if (item->input.value)
 	  free(item->input.value);
-        item->input.value = safemalloc(var_len+1);
+        item->input.buf = var_len+1;
+        item->input.value = safemalloc(item->input.buf);
         strcpy(item->input.value,cp);     /* new value in field */
         /* New value, but don't change length */
         free(var_name);                 /* goto's have their uses */
@@ -816,12 +950,38 @@ static void PutDataInForm(char *cp)
           item->choice.init_on = 0;
           if (strncasecmp(cp, "on", 2) == 0) {
             item->choice.init_on = 1;   /* set default state */
+            free(var_name);             /* goto's have their uses */
+            return;
           }
         }
       }
     } /* end all items in line */
     line = line->next;                  /* go to next line */
   } while (line != &root_line);         /* do all lines */
+  /* You have a matching line, it didn't match an input field,
+     and it didn't match a choice.  I've got it, it may match a
+     selection, in which case we should use the value to
+     set the choices! */
+  for (item = root_item_ptr; item != 0;
+       item = item->header.next) {/* all items */
+    if (item->type == I_SELECT) {     /* selection is good */
+      if (strcasecmp(var_name,item->header.name) == 0) { /* match */
+        /* Now find the choices for this selection... */
+        Item *choice_ptr;
+        for (i = 0;
+             i<item->selection.n;
+             i++) {
+          choice_ptr=item->selection.choices[i];
+          /* if the choice value matches the selection */
+          if (strcmp(choice_ptr->choice.value,var_value) == 0) {
+            choice_ptr->choice.init_on = 1;
+          } else {
+            choice_ptr->choice.init_on = 0;
+          }
+        } /* end all choices */
+      } /* end match */
+    } /* end selection */
+  } /* end all items */
   free(var_name);                       /* not needed now */
 }
 static void ct_Selection(char *cp)
@@ -943,21 +1103,28 @@ static void ct_Command(char *cp)
 			   sizeof(char *) *
 			   cur_button->button.button_array_size);
   }
-  cur_button->button.commands[cur_button->button.n++] = strdup(cp);
+  cur_button->button.commands[cur_button->button.n++] = safestrdup(cp);
 }
 
 /* End of ct_ routines */
 
 /* Init constants with values that can be freed later. */
 static void InitConstants () {
-  color_names[0]=strdup("Light Gray");
-  color_names[1]=strdup("Black");
-  color_names[2]=strdup("Gray50");
-  color_names[3]=strdup("Wheat");
-  font_names[0]=strdup("8x13bold");
-  font_names[1]=strdup("8x13bold");
-  font_names[2]=strdup("8x13bold");
-  screen_background_color=strdup("Light Gray");
+  color_names[0]=safestrdup("Light Gray");
+  color_names[1]=safestrdup("Black");
+  color_names[2]=safestrdup("Gray50");
+  color_names[3]=safestrdup("Wheat");
+  font_names[0]=safestrdup("8x13bold");
+  font_names[1]=safestrdup("8x13bold");
+  font_names[2]=safestrdup("8x13bold");
+  screen_background_color=safestrdup("Light Gray");
+  CF.p_c[input_fore].pointer_color.pixel = WhitePixel(dpy, screen);
+  CF.p_c[input_back].pointer_color.pixel = BlackPixel(dpy, screen);
+  CF.p_c[button_fore].pointer_color.pixel = BlackPixel(dpy, screen);
+  CF.p_c[button_back].pointer_color.pixel = WhitePixel(dpy, screen);
+  /* The in pointer is the reverse of the hover pointer. */
+  CF.p_c[button_in_fore].pointer_color.pixel = WhitePixel(dpy, screen);
+  CF.p_c[button_in_back].pointer_color.pixel = BlackPixel(dpy, screen);
 }
 
 /* read the configuration file */
@@ -1121,7 +1288,7 @@ static void Restart ()
         if (item->input.value_history_ptr == 0) {  /* no history yet */
           item->input.value_history_ptr =
 	    (char **)safecalloc(sizeof(char *), 50);
-          item->input.value_history_ptr[0] = strdup(item->input.value);
+          item->input.value_history_ptr[0] = safestrdup(item->input.value);
           item->input.value_history_count = 1; /* next insertion point */
           myfprintf((stderr,"Initial save of %s in slot 0\n",
                      item->input.value_history_ptr[0]));
@@ -1145,7 +1312,7 @@ static void Restart ()
                          item->input.value_history_count));
             }
             item->input.value_history_ptr[item->input.value_history_count] =
-              strdup(item->input.value); /* save value ptr in array */
+              safestrdup(item->input.value); /* save value ptr in array */
             myfprintf((stderr,"Save of %s in slot %d\n",
                        item->input.value,
                        item->input.value_history_count));
@@ -1201,13 +1368,20 @@ void RedrawFrame ()
 void RedrawText(Item *item)
 {
   int x, y;
+  int len;
+  char *p;
+
   CheckAlloc(item,item->header.dt_ptr); /* alloc colors and fonts needed */
   x = item->header.pos_x + TEXT_SPC;
   y = item->header.pos_y + ( CF.padVText / 2 ) +
     item->header.dt_ptr->dt_font_struct->ascent;
+  len = item->text.n;
+  if ((p = memchr(item->text.value, '\0', len)) != NULL)
+    len = p - item->text.value;
   XDrawString(dpy, CF.frame, item->header.dt_ptr->dt_GC,
-                   x, y, item->text.value,
-                   item->text.n);
+	      x, y, item->text.value, len);
+
+  return;
 }
 
 /* redraw an item */
@@ -1382,10 +1556,11 @@ void RedrawItem (Item *item, int click)
                   xsegs, 4);
     XSetForeground(dpy, item->header.dt_ptr->dt_item_GC,
                    item->header.dt_ptr->dt_colors[c_item_fg]);
-    XDrawString(dpy, item->header.win, item->header.dt_ptr->dt_item_GC,
-		     BOX_SPC + TEXT_SPC,
-		     BOX_SPC + TEXT_SPC + item->header.dt_ptr->dt_font_struct->ascent,
-		     item->button.text, item->button.len);
+    XDrawString(
+      dpy, item->header.win, item->header.dt_ptr->dt_item_GC,
+      BOX_SPC + TEXT_SPC,
+      BOX_SPC + TEXT_SPC + item->header.dt_ptr->dt_font_struct->ascent,
+      item->button.text, item->button.len);
     myfprintf((stderr,"Just put %s into a button\n",
                item->button.text));
     break;
@@ -1446,7 +1621,7 @@ void DoCommand (Item *cmd)
        Should not be a problem, there shouldn't be any more commands
        coming into FvwmForm.  dje */
   }
-  if (cmd->button.key == IB_RESTART) {
+  else if (cmd->button.key == IB_RESTART) {
     Restart();
     for (item = root_item_ptr; item != 0;
          item = item->header.next) {    /* all items */
@@ -1464,47 +1639,104 @@ void DoCommand (Item *cmd)
 static void OpenWindows ()
 {
   int x, y;
+  int gravity = NorthWestGravity;
   Item *item;
-  static XColor xcf, xcb;
   static XSetWindowAttributes xswa;
   static XWMHints wmh = { InputHint, True };
-  static XSizeHints sh = { PPosition | PSize | USPosition | USSize };
+  static XSizeHints sh =
+    { PPosition | PSize | USPosition | USSize | PWinGravity};
   XClassHint myclasshints;
 
-  xc_ibeam = XCreateFontCursor(dpy, XC_xterm);
-  xc_hand = XCreateFontCursor(dpy, XC_hand2);
-  xcf.pixel = GetColor("White");
-  XQueryColor(dpy, Pcmap, &xcf);
-  xcb.pixel = CF.screen_background = (colorset < 0)
+  if (!CF.pointer[input_pointer]) {
+    CF.pointer[input_pointer] = XCreateFontCursor(dpy, XC_xterm);
+  }
+  if (!CF.pointer[button_in_pointer]) {
+    CF.pointer[button_in_pointer] = XCreateFontCursor(dpy, XC_hand2);
+  }
+  if (!CF.pointer[button_pointer]) {
+    CF.pointer[button_pointer] = XCreateFontCursor(dpy,XC_hand2);
+  }
+  CF.screen_background = (colorset < 0)
     ? GetColor(screen_background_color)
     : Colorset[colorset].bg;
-  XQueryColor(dpy, Pcmap, &xcb);
-  XRecolorCursor(dpy, xc_ibeam, &xcf, &xcb);
 
+  if (!CF.p_c[input_back].used) {  /* if not set, use screen b/g */
+    CF.p_c[input_back].pointer_color.pixel = CF.screen_background;
+  }
+  myfprintf((stderr,
+             "screen bg %X, getcolor bg %X, colorset bg %X colorset %d\n",
+             (int)CF.screen_background,
+             (int)GetColor(screen_background_color),
+             (int)Colorset[colorset].bg,
+             (int)colorset));
+  XQueryColor(dpy, Pcmap, &CF.p_c[input_fore].pointer_color);
+  XQueryColor(dpy, Pcmap, &CF.p_c[input_back].pointer_color);
+  XRecolorCursor(dpy, CF.pointer[input_pointer],
+                 &CF.p_c[input_fore].pointer_color,
+                 &CF.p_c[input_back].pointer_color);
+  myfprintf((stderr,"input fore %X, back %X\n",
+          (int)CF.p_c[input_fore].pointer_color.pixel,
+          (int)CF.p_c[input_back].pointer_color.pixel));
+  /* The input cursor is handled differently than the 2 button cursors. */
+  XQueryColor(dpy, Pcmap, &CF.p_c[button_fore].pointer_color);
+  XQueryColor(dpy, Pcmap, &CF.p_c[button_back].pointer_color);
+  XRecolorCursor(dpy, CF.pointer[button_pointer],
+                 &CF.p_c[button_fore].pointer_color,
+                 &CF.p_c[button_back].pointer_color);
+  myfprintf((stderr,"button fore %X, back %X\n",
+          (int)CF.p_c[button_fore].pointer_color.pixel,
+          (int)CF.p_c[button_back].pointer_color.pixel));
+  XQueryColor(dpy, Pcmap, &CF.p_c[button_in_fore].pointer_color);
+  XQueryColor(dpy, Pcmap, &CF.p_c[button_in_back].pointer_color);
+  XRecolorCursor(dpy, CF.pointer[button_in_pointer],
+                 &CF.p_c[button_in_fore].pointer_color,
+                 &CF.p_c[button_in_back].pointer_color);
+  myfprintf((stderr,"button in fore %X, back %X\n",
+          (int)CF.p_c[button_in_fore].pointer_color.pixel,
+          (int)CF.p_c[button_in_back].pointer_color.pixel));
   /* the frame window first */
-  if (CF.have_geom) {
-    if (CF.gx >= 0)
+  if (CF.have_geom)
+  {
+    if (CF.xneg)
+    {
+      x = DisplayWidth(dpy, screen) - CF.max_width + CF.gx;
+      gravity = NorthEastGravity;
+    }
+    else
+    {
       x = CF.gx;
+    }
+    if (CF.yneg)
+    {
+      y = DisplayHeight(dpy, screen) - CF.total_height + CF.gy;
+      gravity = SouthWestGravity;
+    }
     else
-      x = DisplayWidth(dpy, screen) - CF.max_width + CF.gx;
-    if (CF.gy >= 0)
+    {
       y = CF.gy;
-    else
-      y = DisplayHeight(dpy, screen) - CF.total_height + CF.gy;
+    }
+    if (CF.xneg && CF.yneg)
+    {
+      gravity = SouthEastGravity;
+    }
   } else {
-    x = (DisplayWidth(dpy, screen) - CF.max_width) / 2;
-    y = (DisplayHeight(dpy, screen) - CF.total_height) / 2;
+    FScreenCenterOnScreen(
+      NULL, FSCREEN_CURRENT, &x, &y, CF.max_width, CF.total_height);
   }
-  myfprintf((stderr,"going to create window w. bg %s\n",
-             screen_background_color));
   xswa.background_pixel = CF.screen_background;
   xswa.border_pixel = 0;
   xswa.colormap = Pcmap;
+  myfprintf((stderr,
+          "going to create window w. bg %s, b/g pixel %X, black pixel %X\n",
+          screen_background_color,
+          (int)xswa.background_pixel,
+          BlackPixel(dpy, screen)));
   CF.frame = XCreateWindow(dpy, root, x, y, CF.max_width, CF.total_height, 0,
 			   Pdepth, InputOutput, Pvisual,
 			   CWColormap | CWBackPixel | CWBorderPixel, &xswa);
   XSelectInput(dpy, CF.frame,
-               KeyPressMask | ExposureMask | StructureNotifyMask);
+               KeyPressMask | ExposureMask | StructureNotifyMask |
+	       VisibilityChangeMask);
   if (!CF.title) {
     CF.title = MyName+1;
   }
@@ -1513,8 +1745,11 @@ static void OpenWindows ()
   myclasshints.res_name = MyName+1;
   myclasshints.res_class = "FvwmForm";
   XSetClassHint(dpy,CF.frame,&myclasshints);
-  sh.x = x, sh.y = y;
-  sh.width = CF.max_width, sh.height = CF.total_height;
+  sh.x = x;
+  sh.y = y;
+  sh.width = CF.max_width;
+  sh.height = CF.total_height;
+  sh.win_gravity = gravity;
   XSetWMNormalHints(dpy, CF.frame, &sh);
 
   for (item = root_item_ptr; item != 0;
@@ -1529,7 +1764,7 @@ static void OpenWindows ()
 			    0, CF.screen_background,
                             item->header.dt_ptr->dt_colors[c_item_bg]);
       XSelectInput(dpy, item->header.win, ButtonPressMask | ExposureMask);
-      xswa.cursor = xc_ibeam;
+      xswa.cursor = CF.pointer[input_pointer];
       XChangeWindowAttributes(dpy, item->header.win, CWCursor, &xswa);
       if (itemcolorset >= 0)
       {
@@ -1548,7 +1783,7 @@ static void OpenWindows ()
 			    0, CF.screen_background,
                             item->header.dt_ptr->dt_colors[c_item_bg]);
       XSelectInput(dpy, item->header.win, ButtonPressMask | ExposureMask);
-      xswa.cursor = xc_hand;
+      xswa.cursor = CF.pointer[button_pointer];
       XChangeWindowAttributes(dpy, item->header.win, CWCursor, &xswa);
       if (itemcolorset >= 0)
       {
@@ -1569,7 +1804,7 @@ static void OpenWindows ()
                             item->header.dt_ptr->dt_colors[c_item_bg]);
       XSelectInput(dpy, item->header.win,
 		   ButtonPressMask | ExposureMask);
-      xswa.cursor = xc_hand;
+      xswa.cursor = CF.pointer[button_pointer];
       XChangeWindowAttributes(dpy, item->header.win, CWCursor, &xswa);
       if (itemcolorset >= 0)
       {
@@ -1588,7 +1823,7 @@ static void OpenWindows ()
                         &Colorset[(colorset)], Pdepth,
                         root_item_ptr->header.dt_ptr->dt_GC, True);
   }
-  if (preload_yorn == 'n') {            /* if not a preload */
+  if (preload_yorn != 'y') {            /* if not a preload */
     XMapRaised(dpy, CF.frame);
     XMapSubwindows(dpy, CF.frame);
     if (CF.warp_pointer) {
@@ -1677,32 +1912,39 @@ static void ParseActiveMessage(char *buf)
     if(n == colorset || n == itemcolorset) {
       for (item = root_item_ptr; item != 0;
            item = item->header.next) {
-        item->header.dt_ptr->dt_used = 0;
-        if(item->header.dt_ptr->dt_GC) {
-          XFreeGC(dpy,item->header.dt_ptr->dt_GC);
-          item->header.dt_ptr->dt_GC = NULL;
-        }
-        if(item->header.dt_ptr->dt_item_GC) {
-          XFreeGC(dpy,item->header.dt_ptr->dt_item_GC);
-          item->header.dt_ptr->dt_item_GC = NULL;
-        }
-        CheckAlloc(item,item->header.dt_ptr); /* alloc colors and fonts needed */
-        RedrawItem(item, 0);
-        if (itemcolorset >= 0 && item->header.win != 0) {
-          SetWindowBackground(dpy, item->header.win,
-                              item->header.size_x, item->header.size_y,
-                              &Colorset[(itemcolorset)], Pdepth,
-                              item->header.dt_ptr->dt_GC, True);
-        }
-      }
-      if (colorset >= 0)
-      {
-	SetWindowBackground(dpy, CF.frame, CF.max_width, CF.total_height,
-			    &Colorset[(colorset)], Pdepth,
-			    root_item_ptr->header.dt_ptr->dt_GC, True);
-      }
+        if (item->header.dt_ptr) {      /* if item has a drawtable */
+          item->header.dt_ptr->dt_used = 0;
+          if(item->header.dt_ptr->dt_GC) {
+            XFreeGC(dpy,item->header.dt_ptr->dt_GC);
+            item->header.dt_ptr->dt_GC = NULL;
+          }
+          if(item->header.dt_ptr->dt_item_GC) {
+            XFreeGC(dpy,item->header.dt_ptr->dt_item_GC);
+            item->header.dt_ptr->dt_item_GC = NULL;
+          }
+          CheckAlloc(item,item->header.dt_ptr); /* alloc colors/fonts needed */
+          RedrawItem(item, 0);
+          if (itemcolorset >= 0 && item->header.win != 0) {
+            SetWindowBackground(dpy, item->header.win,
+                                item->header.size_x, item->header.size_y,
+                                &Colorset[(itemcolorset)], Pdepth,
+                                item->header.dt_ptr->dt_GC, True);
+          }
+        } /* end item has a drawtable */
+        if (colorset >= 0)
+          {
+            SetWindowBackground(dpy, CF.frame, CF.max_width, CF.total_height,
+                                &Colorset[(colorset)], Pdepth,
+                                root_item_ptr->header.dt_ptr->dt_GC, True);
+          }
+      } /* end all items */
     }
     return;
+  } /* end colorset command */
+  if (strncasecmp(buf, XINERAMA_CONFIG_STRING, sizeof(XINERAMA_CONFIG_STRING)-1)
+      == 0) {
+    FScreenConfigureModule(buf + sizeof(XINERAMA_CONFIG_STRING)-1);
+    return;
   }
   if (strncasecmp(buf, MyName, MyNameLen) != 0) {/* If its not for me */
     return;
@@ -1885,8 +2127,10 @@ int main (int argc, char **argv)
   myfprintf((stderr, "ref == %d\n", (int)ref));
 
   InitPictureCMap(dpy);
+  FScreenInit(dpy);
   /* prevent core dumps if fvwm doesn't provide any colorsets */
   AllocColorset(0);
+  FShapeInit(dpy);
 
   fd_x = XConnectionNumber(dpy);
 
