@@ -17,15 +17,19 @@
 
 
 
-/***********************************************/
-/* Fonction pour RadioButton                    */
-/***********************************************/
+/*********************************************************/
+/* Fonction pour RadioButton / Function for RadioButton  */
+/*********************************************************/
 void InitRadioButton(struct XObj *xobj)
 {
  unsigned long mask;
  XSetWindowAttributes Attr;
  int asc,desc,dir;
  XCharStruct struc;
+#ifdef I18N_MB
+ char **ml;
+ XFontStruct **fs_list;
+#endif
 
  /* Enregistrement des couleurs et de la police */
  if (xobj->colorset >= 0) {
@@ -54,12 +58,20 @@ void InitRadioButton(struct XObj *xobj)
  xobj->gc=fvwmlib_XCreateGC(dpy,xobj->win,0,NULL);
  XSetForeground(dpy,xobj->gc,xobj->TabColor[fore]);
 
- if ((xobj->xfont=XLoadQueryFont(dpy,xobj->font))==NULL)
-  {
-   fprintf(stderr,"Can't load font %s\n",xobj->font);
-  }
- else
-  XSetFont(dpy,xobj->gc,xobj->xfont->fid);
+#ifdef I18N_MB
+ if ((xobj->xfontset=GetFontSetOrFixed(dpy,xobj->font)) == NULL) {
+     fprintf(stderr, "FvwmScript: Couldn't load font. Exiting!\n");
+     exit(1);
+ }
+ XFontsOfFontSet(xobj->xfontset,&fs_list,&ml);
+ xobj->xfont = fs_list[0];
+#else
+ if ((xobj->xfont=GetFontOrFixed(dpy,xobj->font))==NULL) {
+   fprintf(stderr, "FvwmScript: Couldn't load font. Exiting!\n");
+   exit(1);
+ }
+#endif
+ XSetFont(dpy,xobj->gc,xobj->xfont->fid);
 
  XSetLineAttributes(dpy,xobj->gc,1,LineSolid,CapRound,JoinMiter);
 
@@ -198,6 +210,17 @@ void EvtMouseRadioButton(struct XObj *xobj,XButtonEvent *EvtButton)
 
 void EvtKeyRadioButton(struct XObj *xobj,XKeyEvent *EvtKey)
 {
+  KeySym ks;
+  unsigned char buf[10];
+  int j=xobj->height/2+3;
+
+  XLookupString(EvtKey, (char *)buf, sizeof(buf), &ks, NULL);
+  if (ks == XK_Return) {
+    xobj->value=1;
+    XSetForeground(dpy,xobj->gc,xobj->TabColor[fore]);
+    XFillArc(dpy, xobj->win, xobj->gc, 2, j-10, 8, 8, 0*64, 360*64);
+    SendMsg(xobj,SingleClic);
+  }
 }
 
 void ProcessMsgRadioButton(struct XObj *xobj,unsigned long type,unsigned long *body)
