@@ -623,7 +623,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	}
 	else if (!IsItemIndexSticky(&windows,i))
 	{
-	  is_on_desk = (DeskNumber != cfgpacket->desk);
+	  is_on_desk = (DeskNumber == cfgpacket->desk);
 	  is_changing_desk = (Desk != cfgpacket->desk);
 	}
 	UpdateItemIndexDesk(&windows, i, cfgpacket->desk);
@@ -1482,7 +1482,6 @@ void LoopOnEvents(void)
       /* useful because of dynamic style change */
       if (!Event.xconfigure.send_event)
       {
-	PurgeConfigEvents();
 	break;
       }
       else if (Event.xconfigure.x != win_x || Event.xconfigure.y != win_y)
